/*	DOS Stamp Graphic LCD Test Program
	(C)1999 Bagotronix Inc.
	Author: Ivan Baggett

	Written in Borland C v3.0.
	Use this program as an example on how to operate the DOS Stamp with a
	graphic LCD module based on the T6963C controller.
*/

#include <dos.h>
#include <stdio.h>

#include "stdinc.h"
#include "dosstamp.h"

#define PCS0	16		// PCS0# function is on PIO16 pin
#define	LCD_BASE	PCS0_BASE	// base I/O address of LCD
#define	LCD_STATUS	(LCD_BASE+1)
#define	LCD_DATA	LCD_BASE
#define	LCD_COMMAND	(LCD_BASE+1)

#define COMMAND 1
#define DATA 0

char inittbl[][2] = {
	{	COMMAND,	0x80	},
	{	DATA,		0		},
	{	DATA,		0		},
	{	COMMAND,	0x42	},
	{	DATA,		0x28	},
	{	DATA,		0		},
	{	COMMAND,	0x43	},
	{	DATA,		0		},
	{	DATA,		0x17	},
	{	COMMAND,	0x40	},
	{	DATA,		0x28	},
	{	DATA,		0		},
	{	COMMAND,	0x41	},

	{	DATA,		0		},
	{	DATA,		0x17	},
	{	COMMAND,	0x24	},
	{	DATA,		0x24	},
	{	COMMAND,	0xc0	},
	{	DATA,		0x25	},
	{	COMMAND,	0xc0	},
	{	DATA,		0x2e	},
	{	COMMAND,	0xc0	},
	{	COMMAND,	0x94	}
};

BOOLEAN done = FALSE;


WORD waitforkey (void) {
	WORD c;

	while (!kbhit());
	if (!(c = getch()))
		c = getch();
	return c;
}


UWORD LCDstatus (void) {
	return ((UWORD)inportb (LCD_STATUS));
}


void LCDwrite (BOOLEAN op, BYTE c) {
	WORD u;

	while ((LCDstatus() & 0x0003) != 0x0003);	// wait for LCD
	if (op == COMMAND)
		outportb (LCD_COMMAND, c);	// command
	else
		outportb (LCD_DATA, c);	// data
}


void LCDputc (BYTE c) {
	LCDwrite (DATA, c - 0x20);	// the char adjusted for CGROM offset
	LCDwrite (COMMAND, 0xc0);	// data write autoincrement
}


int LCDputs (const char *s) {
	char *t;

	t = (char *) s;							/* copy pointer */
	while (*t)
		LCDputc (*t++);
	return 1;
}


int main (void) {
	WORD c, i;
	UWORD u;

	// set up PIO16 as Simple Bus Interface chip select PCS0#
	disable();
	PIOconfig (PCS0, PIOCFG_NORMAL, 1);
	enable();

	// LCD initialize
	for (i = 0; i < sizeof inittbl / sizeof inittbl[0]; i++) {
		LCDwrite (inittbl[i][0], inittbl[i][1]);
	}

	// clear display RAM
	LCDwrite (DATA, 0);
	LCDwrite (DATA, 0);
	LCDwrite (COMMAND, 0x24);		// address pointer set
	LCDwrite (COMMAND, 0xb0);		// data auto write
	for (i = 0; i < 8192; i++) {
		while ((LCDstatus() & 0x0008) != 0x0008);	// wait for LCD auto write
		LCDwrite (DATA, 0);
	}
	LCDwrite (COMMAND, 0xb2);		// auto reset

	// go to text home
	LCDwrite (DATA, 0);
	LCDwrite (DATA, 0x17);
	LCDwrite (COMMAND, 0x24);		// address pointer set

	// Hello
	LCDputc ('H');
	LCDputc ('e');
	LCDputc ('l');
	LCDputc ('l');
	LCDputc ('o');
	// All characters test
	for (i = 0; i < 256; i++) {
		LCDputc ((BYTE)i);
	}

	// string test
	LCDputs (" LCDputs()");

	printf ("\n\nType to LCD, <space> = done");
	done = FALSE;
	while (!done) {
		while (!kbhit());
			c = getch();
		if (!c)
			getch();
		if (c == ' ')
			done = TRUE;
		LCDputc ((BYTE)c);
	}

	return 0;
}